CodeQL / Analyze (actions) (push) Canceled after 0s
CodeQL / Analyze (python) (push) Canceled after 0s
pre-commit / pre-commit (push) Canceled after 0s
Multi-Arch CI / setup (push) Canceled after 0s
Publish build_manylinux_x86_64 images / publish_build_manylinux_x86_64 (push) Canceled after 0s
Setup (Multi-Arch) / setup (push) Canceled after 0s
Multi-Arch CI / Linux::${{ fromJSON(needs.setup.outputs.linux_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubi10 image / publish_no_rocm_image_ubi10 (push) Canceled after 0s
Multi-Arch CI / Windows::${{ fromJSON(needs.setup.outputs.windows_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Multi-Arch CI / Manifest Diff (push) Canceled after 0s
TheRock Manifest Diff Report / Generate Manifest Diff Report (push) Canceled after 0s
Multi-Arch CI / CI Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_ocl_rt images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_rocgdb images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Unit Tests / Unit Tests :: ubuntu-24.04 (push) Canceled after 0s
Unit Tests / Unit Tests :: windows-2022 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_media (push) Canceled after 0s
Unit Tests / Unit Tests Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_openmpi (push) Canceled after 0s
109 lines
2.4 KiB
Plaintext
109 lines
2.4 KiB
Plaintext
// RUN: %clang_cc1 -std=c++11 -fcuda-is-device -verify=dev,expected -fsyntax-only \
|
|
// RUN: -verify-ignore-unexpected=warning -verify-ignore-unexpected=note %s
|
|
// RUN: %clang_cc1 -std=c++11 -verify -fsyntax-only \
|
|
// RUN: -verify-ignore-unexpected=warning -verify-ignore-unexpected=note %s
|
|
|
|
#include "Inputs/cuda.h"
|
|
|
|
__device__ void device_fn() {
|
|
auto f1 = [&] {};
|
|
f1(); // implicitly __device__
|
|
|
|
auto f2 = [&] __device__ {};
|
|
f2();
|
|
|
|
auto f3 = [&] __host__ {};
|
|
f3(); // expected-error {{no matching function}}
|
|
|
|
auto f4 = [&] __host__ __device__ {};
|
|
f4();
|
|
|
|
// Now do it all again with '()'s in the lambda declarations: This is a
|
|
// different parse path.
|
|
auto g1 = [&]() {};
|
|
g1(); // implicitly __device__
|
|
|
|
auto g2 = [&]() __device__ {};
|
|
g2();
|
|
|
|
auto g3 = [&]() __host__ {};
|
|
g3(); // expected-error {{no matching function}}
|
|
|
|
auto g4 = [&]() __host__ __device__ {};
|
|
g4();
|
|
|
|
// Once more, with the '()'s in a different place.
|
|
auto h1 = [&]() {};
|
|
h1(); // implicitly __device__
|
|
|
|
auto h2 = [&] __device__ () {};
|
|
h2();
|
|
|
|
auto h3 = [&] __host__ () {};
|
|
h3(); // expected-error {{no matching function}}
|
|
|
|
auto h4 = [&] __host__ __device__ () {};
|
|
h4();
|
|
}
|
|
|
|
// Behaves identically to device_fn.
|
|
__global__ void kernel_fn() {
|
|
auto f1 = [&] {};
|
|
f1(); // implicitly __device__
|
|
|
|
auto f2 = [&] __device__ {};
|
|
f2();
|
|
|
|
auto f3 = [&] __host__ {};
|
|
f3(); // expected-error {{no matching function}}
|
|
|
|
auto f4 = [&] __host__ __device__ {};
|
|
f4();
|
|
|
|
// No need to re-test all the parser contortions we test in the device
|
|
// function.
|
|
}
|
|
|
|
__host__ void host_fn() {
|
|
auto f1 = [&] {};
|
|
f1(); // implicitly __host__ (i.e., no magic)
|
|
|
|
auto f2 = [&] __device__ {};
|
|
f2(); // expected-error {{no matching function}}
|
|
|
|
auto f3 = [&] __host__ {};
|
|
f3();
|
|
|
|
auto f4 = [&] __host__ __device__ {};
|
|
f4();
|
|
}
|
|
|
|
__host__ __device__ void hd_fn() {
|
|
auto f1 = [&] {};
|
|
f1(); // implicitly __host__ __device__
|
|
|
|
auto f2 = [&] __device__ {};
|
|
f2();
|
|
#ifndef __CUDA_ARCH__
|
|
// expected-error@-2 {{reference to __device__ function}}
|
|
#endif
|
|
|
|
auto f3 = [&] __host__ {};
|
|
f3();
|
|
#ifdef __CUDA_ARCH__
|
|
// expected-error@-2 {{reference to __host__ function}}
|
|
#endif
|
|
|
|
auto f4 = [&] __host__ __device__ {};
|
|
f4();
|
|
}
|
|
|
|
// The special treatment above only applies to lambdas.
|
|
__device__ void foo() {
|
|
struct X {
|
|
void foo() {}
|
|
};
|
|
X x;
|
|
x.foo(); // dev-error {{reference to __host__ function 'foo' in __device__ function}}
|
|
}
|